import uno
from com.sun.star.sheet.ConditionOperator import NONE,EQUAL,NOT_EQUAL,GREATER,GREATER_EQUAL,LESS,LESS_EQUAL,BETWEEN,NOT_BETWEEN,FORMULA

readCF = 'readCF'
writeCF = 'writeCF'

g_OP = {
    '=' : EQUAL,
    '<>': NOT_EQUAL,
    '<' : LESS,
    '<=': LESS_EQUAL,
    '>' : GREATER,
    '>=': GREATER_EQUAL,
    '<=>': BETWEEN,
    '><': NOT_BETWEEN,
    'f(x)': FORMULA
    }

def getActiveCell(oView):
    '''Desparately missing in API. We extract from view data.'''
    sData = oView.getViewData()
    oSheet = oView.getActiveSheet()
    as1  = sData.split(";")
    lSheet = long(as1[1])
    sDum = as1[lSheet +3]
    delim = '/' in sDum and '/' or '+'
    as2 = sDum.split(delim)
    lCol = (as2[0])
    lRow = (as2[1])
    return oSheet.getCellByPosition(lCol,lRow)

def _getConditionalEntry(sF1,sF2,lOp,oSrc):
    aProps = list()
    for i in range(4):
        aProps.append(uno.createUnoStruct('com.sun.star.beans.PropertyValue'))

    aProps[0].Name = "Formula1"
    aProps[0].Value = sF1
    aProps[1].Name = "Formula2"
    aProps[1].Value = sF2
    aProps[2].Name = "Operator"
    aProps[2].Value = lOp
    aProps[3].Name = "SourcePosition"
    aProps[3].Value = oSrc
    return tuple(aProps)

def getRangeByAddress(obj,oAddr):
    """Get a sub-range within given doc, sheet or range by given range address.
    If a sheet or range is passed then the sheet of the address is ignored."""
    if obj.supportsService("com.sun.star.sheet.SpreadsheetDocument"):
        oParent = obj.Sheets.getByIndex(oAddr.Sheet)
    else:
        oParent = obj
        
    try:
        return oParent.getCellRangeByPosition(
            oAddr.StartColumn,
            oAddr.StartRow,
            oAddr.EndColumn,
            oAddr.EndRow
            )
    except:
        return None
    

def print_StyleNames():
    doc = XSCRIPTCONTEXT.getDocument()
    view = doc.getCurrentController()
    oAC = getActiveCell(view)
    sh = oAC.getSpreadsheet()
    adr = oAC.getCellAddress()
    oFamily = doc.StyleFamilies.getByName("CellStyles")
    cnt = oFamily.getCount()
    for c in range(0, cnt -1):
        style = oFamily.getByIndex(c)
        # localized name:
        sh.getCellByPosition(adr.Column, adr.Row +c).setString(style.DisplayName)

def print_Operators():
    aOP = ('=', '<>', '<', '<=', '>', '>=', '<=>','><','f(x)')
    doc = XSCRIPTCONTEXT.getDocument()
    view = doc.getCurrentController()
    oAC = getActiveCell(view)
    sh = oAC.getSpreadsheet()
    adr = oAC.getCellAddress()
    for c in range(0, len(aOP)):
        sh.getCellByPosition(adr.Column, adr.Row +c).setString(aOP[c])

def getOPString(e):
    for x in g_OP.items():
        if x[1] == e:
            return x[0]

def read_ConditionalFormats():
    doc = XSCRIPTCONTEXT.getDocument()
    view = doc.getCurrentController()
    oAC = getActiveCell(view)
    oRelPos = oAC.getCellAddress()
    oSel = view.getSelection()
    oCF = oSel.ConditionalFormatLocal
    oSrc = doc.NamedRanges.getByName(readCF).getReferredCells()
    oCF.clear()
    # we've got to get the *string* of each single cell:
    for i in range(oSrc.Rows.getCount()):
        fml1 = oSrc.getCellByPosition(2, i).getString()
        fml2 = '0'
        style = oSrc.getCellByPosition(0, i).getString()
        op = fml1 and g_OP.get(oSrc.getCellByPosition(1, i).getString(), NONE)
        if op == NONE:
            continue
        elif fml2 and op in (BETWEEN, NOT_BETWEEN,):
            fml2 = oSrc.getCellByPosition(3, i).getString()

        # JDW 15 Jan 2009
        # Strings didn't work so this is a work-around
        fml1_altered = fml1
        try:
            float(fml1)
        except:
            # probably text, not a value
            fml1_altered = '"' + fml1 + '"'
        oCE = _getConditionalEntry(fml1_altered,fml2,op,oRelPos)
        oCF.addNew(oCE)
        oCF.getByIndex(i).setStyleName(style)
        i += 1
    oSel.ConditionalFormatLocal = oCF

def write_ConditionalFormats():
    global g_OP
    doc = XSCRIPTCONTEXT.getDocument()
    view = doc.getCurrentController()
    oAC = getActiveCell(view)
    oCF = oAC.ConditionalFormatLocal
    cnt = oCF.getCount()
    adr = doc.NamedRanges.getByName(writeCF).ReferredCells.getRangeAddress()
    adr.EndColumn = adr.StartColumn +3
    adr.EndRow = adr.StartRow +cnt -1
    aTgt = getRangeByAddress(doc, adr)
    lst = []
    for c in range(cnt):
        cf = oCF.getByIndex(c)
        st = cf.getStyleName()
        op = getOPString(cf.getOperator())
        fml1 = cf.getFormula1()
        fml2 = op in (BETWEEN, NOT_BETWEEN) and cf.getFormula2() or ''
        lst.append((st, op, fml1, fml2,))

    aTgt.setDataArray(tuple(lst))
        
g_exportedScripts = read_ConditionalFormats,write_ConditionalFormats,print_StyleNames, print_Operators,
